@@ -91,48 +91,3 @@ bootstrap.backend = function(data, statistic, R, m, algorithm,
 
 }#BOOTSTRAP.BACKEND
 
-# model averaging for bootstrapped network structures.
-averaged.network.backend = function(strength, nodes, threshold) {
-
-  e = empty.graph(nodes)
-
-  # arcs with a strength of one should always be selected, regardless of
-  # the threshold.
-  significant = (strength$strength > threshold) | (strength$strength == 1)
-
-  # filter also the direction if present in the bn.strength object.
-  if ("direction" %in% names(strength))
-    significant = significant & (strength$direction >= 0.5)
-
-  # nothing to see, move along.
-  if (!any(significant))
-    return(e)
-
-  candidate.arcs = as.matrix(strength[significant, c("from", "to"), drop = FALSE])
-
-  if (all(which.undirected(candidate.arcs))) {
-
-    # update the arcs of the network, no cycles.
-    e$arcs = candidate.arcs
-
-  }#THEN
-  else {
-
-    # update the arcs of the network, minding cycles.
-    e$arcs = .Call(call_smart_network_averaging,
-                   arcs = candidate.arcs,
-                   nodes = nodes,
-                   weights = strength$strength[significant])
-
-  }#ELSE
-
-  # update the network structure.
-  e$nodes = cache.structure(nodes, arcs = e$arcs)
-  # add back illegal arcs, so that cpdag() works correctly.
-  if ("illegal" %in% names(attributes(strength)))
-    e$learning$illegal = attr(strength, "illegal")
-
-  return(e)
-
-}#AVERAGED.NETWORK.BACKEND
-
