@@ -1,50 +1,6 @@
 
-fast.incremental.association.optimized = function(x, whitelist, blacklist,
-  test, alpha, B, max.sx = ncol(x), strict, complete, debug = FALSE) {
-
-  nodes = names(x)
-  mb2 = mb = list()
-
-  # 1. [Compute Markov Blankets]
-  for (node in nodes) {
-
-    backtracking = unlist(sapply(mb, function(x){ node %in% x  }))
-
-    mb[[node]] = fast.ia.markov.blanket(node, data = x, nodes = nodes,
-         alpha = alpha, B = B, whitelist = whitelist, blacklist = blacklist,
-         backtracking = backtracking, test = test, max.sx = max.sx,
-         complete = complete, debug = debug)
-
-  }#FOR
-
-  # check markov blankets for consistency.
-  mb = bn.recovery(mb, nodes = nodes, strict = strict, mb = TRUE, debug = debug)
-
-  # 2. [Compute Graph Structure]
-  for (node in nodes) {
-
-    backtracking = unlist(sapply(mb2, function(x){ node %in% x[["nbr"]]  }))
-
-    # save results in a copy of mb.
-    mb2[[node]] = neighbour(node, mb = mb, data = x, alpha = alpha,
-         B = B, whitelist = whitelist, blacklist = blacklist,
-         backtracking = backtracking, test = test, max.sx = max.sx,
-         complete = complete, debug = debug)
-
-  }#FOR
-
-  # update mb with the results of neighbour().
-  mb = mb2
-
-  # check neighbourhood sets for consistency.
-  mb = bn.recovery(mb, nodes = nodes, strict = strict, debug = debug)
-
-  return(mb)
-
-}#FAST.INCREMENTAL.ASSOCIATION.OPTIMIZED
-
 fast.incremental.association = function(x, cluster = NULL, whitelist,
-  blacklist, test, alpha, B, max.sx = ncol(x), strict, complete, debug = FALSE) {
+  blacklist, test, alpha, B, max.sx = ncol(x), complete, debug = FALSE) {
 
   nodes = names(x)
 
@@ -56,7 +12,7 @@ fast.incremental.association = function(x, cluster = NULL, whitelist,
   names(mb) = nodes
 
   # check markov blankets for consistency.
-  mb = bn.recovery(mb, nodes = nodes, strict = strict, mb = TRUE, debug = debug)
+  mb = bn.recovery(mb, nodes = nodes, mb = TRUE, debug = debug)
 
   # 2. [Compute Graph Structure]
   mb = smartSapply(cluster, as.list(nodes), neighbour, mb = mb, data = x,
@@ -65,18 +21,17 @@ fast.incremental.association = function(x, cluster = NULL, whitelist,
   names(mb) = nodes
 
   # check neighbourhood sets for consistency.
-  mb = bn.recovery(mb, nodes = nodes, strict = strict, debug = debug)
+  mb = bn.recovery(mb, nodes = nodes, debug = debug)
 
   return(mb)
 
 }#FAST.INCREMENTAL.ASSOCIATION
 
 fast.ia.markov.blanket = function(x, data, nodes, alpha, B, whitelist,
-  blacklist, start = character(0), backtracking = NULL, test, max.sx = ncol(x),
-  complete, debug = FALSE) {
+  blacklist, start = character(0), test, max.sx = ncol(x), complete,
+  debug = FALSE) {
 
   nodes = nodes[nodes != x]
-  known.good = known.bad = c()
   whitelisted = nodes[sapply(nodes,
           function(y) { is.whitelisted(whitelist, c(x, y), either = TRUE) })]
   mb = start
@@ -101,29 +56,6 @@ fast.ia.markov.blanket = function(x, data, nodes, alpha, B, whitelist,
   # blacklist is not checked, not all nodes in a markov blanket must be
   # neighbours.
 
-  # use backtracking for a further screening of the nodes to be checked.
-  if (!is.null(backtracking)) {
-
-    # nodes whose markov blanket includes this node are included, because
-    # X \in MB(Y) <=> Y \in MB(X); they can be removed in the backward phase
-    # later on if they are false positives.
-    known.good = names(backtracking[backtracking])
-    mb = unique(c(mb, known.good))
-
-    # and vice versa X \not\in MB(Y) <=> Y \not\in MB(X)
-    known.bad = names(backtracking[!backtracking])
-
-    if (debug) {
-
-      cat("    * known good (backtracking): '", known.good, "'.\n")
-      cat("    * known bad (backtracking): '", known.bad, "'.\n")
-      cat("    * nodes still to be tested for inclusion: '",
-        nodes[nodes %!in% mb], "'.\n")
-
-    }#THEN
-
-  }#THEN
-
   repeat {
 
     # stop if there are no nodes left, or if we cannot add any more nodes
@@ -230,7 +162,7 @@ fast.ia.markov.blanket = function(x, data, nodes, alpha, B, whitelist,
       cat("  * checking nodes for exclusion.\n")
 
     # whitelisted nodes are neighbours, they cannot be removed from the
-    # markov blanket; on the other hand, known.good nodes from backtracking
+    # markov blanket.
     fixed = whitelisted[whitelisted != ""]
 
     pv = roundrobin.test(x = x, z = mb, fixed = fixed, data = data, test = test,
