@@ -1,50 +1,6 @@
 
-grow.shrink.optimized = function(x, whitelist, blacklist, test, alpha,
-  B, max.sx = ncol(x), strict, complete, debug = FALSE) {
-
-  nodes = names(x)
-  mb2 = mb = list()
-
-  # 1. [Compute Markov Blankets]
-  for (node in nodes) {
-
-    backtracking = unlist(sapply(mb, function(x){ node %in% x  }))
-
-    mb[[node]] = gs.markov.blanket(node, data = x, nodes = nodes,
-         alpha = alpha, B = B, whitelist = whitelist, blacklist = blacklist,
-         backtracking = backtracking, test = test, max.sx = max.sx,
-         complete = complete, debug = debug)
-
-  }#FOR
-
-  # check markov blankets for consistency.
-  mb = bn.recovery(mb, nodes = nodes, strict = strict, mb = TRUE, debug = debug)
-
-  # 2. [Compute Graph Structure]
-  for (node in nodes) {
-
-    backtracking = unlist(sapply(mb2, function(x){ node %in% x[["nbr"]]  }))
-
-    # save results in a copy of mb.
-    mb2[[node]] = neighbour(node, mb = mb, data = x, alpha = alpha,
-         B = B, whitelist = whitelist, blacklist = blacklist,
-         backtracking = backtracking, test = test, max.sx = max.sx,
-         complete = complete, debug = debug)
-
-  }#FOR
-
-  # update mb with the results of neighbour().
-  mb = mb2
-
-  # check neighbourhood sets for consistency.
-  mb = bn.recovery(mb, nodes = nodes, strict = strict, debug = debug)
-
-  return(mb)
-
-}#GROW.SHRINK.OPTIMIZED
-
 grow.shrink = function(x, cluster = NULL, whitelist, blacklist, test, alpha, B,
-  strict, max.sx = ncol(x), complete, debug = FALSE) {
+  max.sx = ncol(x), complete, debug = FALSE) {
 
   nodes = names(x)
 
@@ -56,7 +12,7 @@ grow.shrink = function(x, cluster = NULL, whitelist, blacklist, test, alpha, B,
   names(mb) = nodes
 
   # check markov blankets for consistency.
-  mb = bn.recovery(mb, nodes = nodes, strict = strict, mb = TRUE, debug = debug)
+  mb = bn.recovery(mb, nodes = nodes, mb = TRUE, debug = debug)
 
   # 2. [Compute Graph Structure]
   mb = smartSapply(cluster, as.list(nodes), neighbour, mb = mb, data = x,
@@ -65,18 +21,16 @@ grow.shrink = function(x, cluster = NULL, whitelist, blacklist, test, alpha, B,
   names(mb) = nodes
 
   # check neighbourhood sets for consistency.
-  mb = bn.recovery(mb, nodes = nodes, strict = strict, debug = debug)
+  mb = bn.recovery(mb, nodes = nodes, debug = debug)
 
   return(mb)
 
 }#GROW.SHRINK
 
 gs.markov.blanket = function(x, data, nodes, alpha, B, whitelist, blacklist,
-  start = character(0), backtracking = NULL, test, max.sx = ncol(x), complete,
-  debug = FALSE) {
+  start = character(0), test, max.sx = ncol(x), complete, debug = FALSE) {
 
   nodes = nodes[nodes != x]
-  known.good = known.bad = c()
   whitelisted = nodes[sapply(nodes,
           function(y) { is.whitelisted(whitelist, c(x, y), either = TRUE) })]
   mb = start
@@ -100,32 +54,6 @@ gs.markov.blanket = function(x, data, nodes, alpha, B, whitelist, blacklist,
   # blacklist is not checked, not all nodes in a markov blanket must be
   # neighbours.
 
-  # use backtracking for a further screening of the nodes to be checked.
-  if (!is.null(backtracking)) {
-
-    # nodes whose markov blanket includes this node are included, because
-    # X \in MB(Y) <=> Y \in MB(X); they can be removed in the backward phase
-    # later on if they are false positives.
-    known.good = names(backtracking[backtracking])
-    mb = unique(c(mb, known.good))
-
-    # known.good nodes are not to be checked for inclusion, and "mb" is used
-    # for the shrinking phase instead of "nodes", we can just remove them.
-    nodes = nodes[nodes %!in% known.good]
-
-    # and vice versa X \not\in MB(Y) <=> Y \not\in MB(X)
-    known.bad = names(backtracking[!backtracking])
-
-    if (debug) {
-
-      cat("    * known good (backtracking): '", known.good, "'.\n")
-      cat("    * known bad (backtracking): '", known.bad, "'.\n")
-      cat("    * nodes still to be tested for inclusion: '", nodes, "'.\n")
-
-    }#THEN
-
-  }#THEN
-
   # grow phase.
   repeat {
 
@@ -185,8 +113,7 @@ gs.markov.blanket = function(x, data, nodes, alpha, B, whitelist, blacklist,
   }#REPEAT
 
   # whitelisted nodes are neighbours, they cannot be removed from the
-  # markov blanket; on the other hand, known.good nodes from backtracking
-  # should be checked to remove false positives.
+  # markov blanket.
   nodes = mb[mb %!in% whitelisted]
 
   # shrinking phase.
