@@ -1,42 +1,6 @@
 
-maxmin.pc.optimized = function(x, whitelist, blacklist, test, alpha, B,
-    max.sx = ncol(x), strict, complete, debug = FALSE) {
-
-  nodes = names(x)
-  mb = list()
-
-  for (node in nodes) {
-
-    backtracking = unlist(sapply(mb, function(x){ node %in% x$nbr }))
-
-    # 1. [Forward Phase (I)]
-    mb[[node]] = maxmin.pc.forward.phase(node, data = x, nodes = nodes,
-         alpha = alpha, B = B, whitelist = whitelist, blacklist = blacklist,
-         backtracking = backtracking, test = test, max.sx = max.sx,
-         optimized = TRUE, complete = complete, debug = debug)
-
-    # 2. [Backward Phase (II)]
-    mb[[node]] = neighbour(node, mb = mb, data = x, alpha = alpha,
-         B = B, whitelist = whitelist, blacklist = blacklist,
-         backtracking = backtracking, test = test, max.sx = max.sx,
-         markov = FALSE, complete = complete, debug = debug)
-
-  }#FOR
-
-  # make up a set of believable Markov blankets, using all the nodes within
-  # distance 2 from the target node (which is a superset).
-  for (node in nodes)
-    mb[[node]]$mb = fake.markov.blanket(mb, node)
-
-  # check neighbourhood sets for consistency.
-  mb = bn.recovery(mb, nodes = nodes, strict = strict, debug = debug)
-
-  return(mb)
-
-}#MAXMIN.PC.OPTIMIZED
-
 maxmin.pc = function(x, cluster = NULL, whitelist, blacklist, test, alpha, B,
-  strict, max.sx = ncol(x), complete, debug = FALSE) {
+  max.sx = ncol(x), complete, debug = FALSE) {
 
   nodes = names(x)
 
@@ -44,7 +8,7 @@ maxmin.pc = function(x, cluster = NULL, whitelist, blacklist, test, alpha, B,
   mb = smartSapply(cluster, as.list(nodes), maxmin.pc.forward.phase, data = x,
          nodes = nodes, alpha = alpha, B = B, whitelist = whitelist,
          blacklist = blacklist, test = test, max.sx = max.sx,
-         optimized = FALSE, complete = complete, debug = debug)
+         complete = complete, debug = debug)
   names(mb) = nodes
 
   # 2. [Backward Phase (II)]
@@ -60,18 +24,16 @@ maxmin.pc = function(x, cluster = NULL, whitelist, blacklist, test, alpha, B,
     mb[[node]]$mb = fake.markov.blanket(mb, node)
 
   # check neighbourhood sets for consistency.
-  mb = bn.recovery(mb, nodes = nodes, strict = strict, debug = debug)
+  mb = bn.recovery(mb, nodes = nodes, debug = debug)
 
   return(mb)
 
 }#MAXMIN.PC
 
 maxmin.pc.forward.phase = function(x, data, nodes, alpha, B, whitelist,
-  blacklist, backtracking = NULL, test, max.sx = ncol(x), optimized = TRUE,
-  complete, debug = FALSE) {
+  blacklist, test, max.sx = ncol(x), complete, debug = FALSE) {
 
   nodes = nodes[nodes != x]
-  known.good = known.bad = c()
   whitelisted = nodes[sapply(nodes,
           function(y) { is.whitelisted(whitelist, c(x, y), either = TRUE) })]
   blacklisted = nodes[sapply(nodes,
@@ -92,33 +54,6 @@ maxmin.pc.forward.phase = function(x, data, nodes, alpha, B, whitelist,
   cpc = whitelisted
   nodes = nodes[nodes %!in% c(cpc, blacklisted)]
 
-  # use backtracking for a further screening of the nodes to be checked.
-  if (!is.null(backtracking) && optimized) {
-
-    # nodes whose neighbourhood includes this node are included, because
-    # X \in NBR(Y) <=> Y \in NBR(X); they can be removed in the backward
-    # phase later on if they are false positives.
-    known.good = names(backtracking[backtracking])
-    cpc = unique(c(cpc, known.good))
-
-    # and vice versa X not adiacent to Y <=> Y not adiacent to X
-    known.bad = names(backtracking[!backtracking])
-
-    # from empirical observations, MMPC has very few false negatives and
-    # therefore it is fairly safe not to test known.bad nodes.
-    nodes  = nodes[nodes %!in% known.bad]
-
-    if (debug) {
-
-      cat("    * known good (backtracking): '", known.good, "'.\n")
-      cat("    * known bad (backtracking): '", known.bad, "'.\n")
-      cat("    * nodes still to be tested for inclusion: '",
-        nodes[nodes %!in% cpc], "'.\n")
-
-    }#THEN
-
-  }#THEN
-
   # phase I (stepwise forward selection)
   repeat {
 
@@ -127,8 +62,8 @@ maxmin.pc.forward.phase = function(x, data, nodes, alpha, B, whitelist,
       break
 
     # do not check nodes which have a p-value above the alpha threshold, as
-    # it can only increase; also do not check 'known bad' ones.
-    to.be.checked = setdiff(names(which(association < alpha)), c(cpc, known.bad))
+    # it can only increase.
+    to.be.checked = setdiff(names(which(association < alpha)), cpc)
 
     # get an association measure for each of the available nodes.
     association = sapply(to.be.checked, maxmin.pc.heuristic.optimized, y = x,
