@@ -54,13 +54,12 @@ extern SEXP is_dag(SEXP, SEXP);
 extern SEXP is_listed(SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP is_pdag_acyclic(SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP is_row_equal(SEXP, SEXP);
-extern SEXP joint_discretize(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
+extern SEXP joint_discretize(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP lw_weights(SEXP, SEXP, SEXP, SEXP);
 extern SEXP mappred(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
-extern SEXP marginal_discretize(SEXP, SEXP, SEXP, SEXP);
+extern SEXP marginal_discretize(SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP match_brace(SEXP, SEXP, SEXP, SEXP);
 extern SEXP mean_strength(SEXP, SEXP, SEXP);
-extern SEXP mi(SEXP, SEXP, SEXP, SEXP);
 extern SEXP minimal_data_frame(SEXP);
 extern SEXP minimal_table(SEXP, SEXP);
 extern SEXP naivepred(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
