@@ -6,27 +6,34 @@
  * negatives. */
 
 /* determine whether a graph is DAG or a PDAG/UG. */
-SEXP is_dag(SEXP arcs, SEXP nnodes) {
+SEXP is_dag(SEXP arcs, SEXP nodes) {
 
-int i = 0, nrows = length(arcs)/2, n = INT(nnodes);
-int *a = INTEGER(arcs);
+int i = 0, nrow = length(arcs)/2, n = LENGTH(nodes);
+int *a = NULL;
 short int *checklist = NULL;
+SEXP try;
+
+  /* match the node labels in the arc set. */
+  PROTECT(try = match(nodes, arcs, 0));
+  a = INTEGER(try);
 
   /* allocate and initialize the checklist. */
   checklist = Calloc1D(UPTRI_MATRIX(n), sizeof(short int));
 
-  for (i = 0; i < nrows; i++) {
+  for (i = 0; i < nrow; i++) {
 
-    if (checklist[UPTRI(a[CMC(i, 0, nrows)], a[CMC(i, 1, nrows)], n)] == 0) {
+    if (checklist[UPTRI(a[CMC(i, 0, nrow)], a[CMC(i, 1, nrow)], n)] == 0) {
 
       /* this arc is not present in the checklist; add it. */
-      checklist[UPTRI(a[CMC(i, 0, nrows)], a[CMC(i, 1, nrows)], n)] = 1;
+      checklist[UPTRI(a[CMC(i, 0, nrow)], a[CMC(i, 1, nrow)], n)] = 1;
 
     }/*THEN*/
     else {
 
       /* this arc or its opposite already present in the checklist; the graph
        * has at least an undirected arc, so return FALSE. */
+      UNPROTECT(1);
+
       Free1D(checklist);
 
       return ScalarLogical(FALSE);
@@ -35,6 +42,8 @@ short int *checklist = NULL;
 
   }/*FOR*/
 
+  UNPROTECT(1);
+
   Free1D(checklist);
 
   return ScalarLogical(TRUE);
